import logging
import os
import shutil

import yaml  # type: ignore
from jinja2 import Environment, FileSystemLoader


class HomepageGenerator:
    """
    Class to generate a custom homepage from template files. The template files can be created
    with the `DirectoryBuilder` class.
    """

    def __init__(
        self, template_dir: str = "template", settings_yaml: "str" = "settings.yaml", output_dir: str = "public"
    ) -> None:
        self.settings_yaml = settings_yaml
        self.output_dir = output_dir
        self.template_dir = template_dir
        self.env = Environment(loader=FileSystemLoader(template_dir))

    def build(self) -> None:
        self._verify_required_files_exist()
        self._clear_output_dir_if_exists()
        self._create_output_dir()
        self._copy_static_dir()
        self._render_page()
        self._add_images_list_to_js()
        logging.info(f"Page built. The resulting homepage can be found at {self.output_dir}/homepage.html")

    def _verify_required_files_exist(self) -> None:
        expected_files = [
            self.settings_yaml,
            f"{self.template_dir}/_homepage.html",
            f"{self.template_dir}/static/_stylesheet.css",
            f"{self.template_dir}/static/homepage.js",
        ]
        files_found = [os.path.exists(file) for file in expected_files]
        if not all(files_found):
            raise ValueError(
                "Not all required template files were found! Did you initialize the directory with `homepage init` first?"
            )

    def _clear_output_dir_if_exists(self) -> None:
        if os.path.isdir(self.output_dir):
            try:
                shutil.rmtree(self.output_dir)
            except Exception as e:
                logging.info(e)
                raise ValueError(f"Error encountered while cleaning {self.output_dir} directory.")

    def _create_output_dir(self) -> None:
        try:
            os.mkdir(self.output_dir)
        except Exception as e:
            logging.info(e)
            raise ValueError(f"Encountered an error while trying to create the '{self.output_dir}' directory")

    def _copy_static_dir(self) -> None:
        try:
            shutil.copytree(f"{self.template_dir}/static", f"{self.output_dir}/static")
            os.remove(f"{self.output_dir}/static/_stylesheet.css")
        except Exception as e:
            logging.info(e)
            raise ValueError("Encountered an error while copying static files.")

    def _render_page(self) -> None:

        with open(self.settings_yaml, "rt") as f:
            settings = yaml.safe_load(f.read())

        template = self.env.get_template("_homepage.html")
        with open(f"{self.output_dir}/homepage.html", "w+") as file:
            html = template.render(settings=settings)
            file.write(html)

        template = self.env.get_template("static/_stylesheet.css")
        with open(f"{self.output_dir}/static/stylesheet.css", "w+") as file:
            html = template.render(settings=settings)
            file.write(html)

    def _add_images_list_to_js(self) -> None:
        """
        Scan for any images and add this as a list to the homepage.js files, so JavaScript can randomly pick
        one to display when the page is opened.
        """
        images = [f'"static/images/{image}"' for image in os.listdir("public/static/images")]

        with open("public/static/homepage.js", "r+") as f:
            content = f.read()
            f.seek(0, 0)
            line = f'var images = new Array({",".join(images)});'
            f.write(line.rstrip("\r\n") + "\n" + content)



import argparse
import logging
import sys

from simple_homepage.directory_builder import DirectoryBuilder
from simple_homepage.homepage_generator import HomepageGenerator

logging.basicConfig(level=logging.INFO, handlers=[logging.StreamHandler()], format="%(message)s")


def cli() -> None:
    """
    Wrapper so commands do not not return the object.
    """
    CommandLineInterface()


class CommandLineInterface:
    def __init__(self) -> None:
        parser = argparse.ArgumentParser(
            description="Generate a simple homepage",
            usage="""homepage <command> [<args>]

            The available commands:
            init     Initialize the file structure for the homepage
            build    Build the homepage from template files and settings.yaml
            """,
        )

        parser.add_argument("command", help="Subcommand to run")
        args = parser.parse_args(sys.argv[1:2])
        if not hasattr(self, args.command):
            print("Unrecognized command")
            parser.print_help()
            exit(1)
        # use dispatch pattern to invoke method with same name
        getattr(self, args.command)()

    def init(self) -> None:
        parser = argparse.ArgumentParser(description="Initialize the file structure for the homepage")
        parser.add_argument(
            "--dark",
            dest="dark",
            action="store_true",
            help="""Optional. By default the template is initialized in light mode.
            Add this flag to initialize the page in dark mode.""",
        )
        parser.add_argument(
            "--dir",
            dest="dir",
            type=str,
            help="""Optional. Name of a directory (relative to the current directory) to create and place the templates in.
            If not specified, the template files will be placed in the current directory. """,
        )
        parser.add_argument(
            "--overwrite",
            dest="overwrite",
            action="store_true",
            help="""Optional. Ignore errors and overwrite existing files if they are found.""",
        )
        args = parser.parse_args(sys.argv[2:])
        dict_args = vars(args)
        try:
            DirectoryBuilder(
                dark=dict_args["dark"], directory=dict_args["dir"], overwrite=dict_args["overwrite"]
            ).build()
        except (FileExistsError) as e:
            logging.info(e)
            logging.info(
                "Run the command with the `--overwrite` flag to ignore these errors and overwrite existing files."
            )

    def build(self) -> None:
        parser = argparse.ArgumentParser(
            description="Build the homepage from template files and settings.yaml"
        )  # noqa: F841
        HomepageGenerator().build()



import logging
import os
import shutil
from pathlib import Path
from typing import Union

import oyaml  # type: ignore
import pkg_resources  # type: ignore


class DirectoryBuilder:
    """
    Class to populate the directory with the template files required to build a custom homepage.
    """

    def __init__(self, dark: bool = False, directory: Union[str, None] = None, overwrite: bool = False) -> None:
        self.dark = dark
        self.directory = directory
        self.directory_path = Path(os.getcwd()) / directory if directory else Path(os.getcwd())
        self.overwrite = overwrite

    def build(self) -> None:

        if self.directory:
            self._create_directory()
        if not self.overwrite:
            self._verify_that_template_files_dont_exist()
        self._copy_files_from_package()
        self._rename_placeholder_images()
        if self.dark:
            self._change_colors_in_settings_to_dark_mode()

        logging.info("Template files and settings.yaml created.")

    def _create_directory(self) -> None:
        if not os.path.exists(self.directory):  # type: ignore
            os.mkdir(self.directory)  # type: ignore
        else:
            if not self.overwrite:
                raise OSError(f"A folder '{self.directory}' already exists within the current directory!")

    def _verify_that_template_files_dont_exist(self) -> None:
        if os.path.isdir(self.directory_path / "template") or os.path.isfile(self.directory_path / "settings.yaml"):
            raise FileExistsError("The directory already contains template files.")

    def _copy_files_from_package(self) -> None:
        """
        Copy the files in the 'files' directory from the package contents to the local client.
        """
        try:
            files_location = pkg_resources.resource_filename(__name__, "files")
            shutil.copytree(files_location, str(self.directory_path), dirs_exist_ok=True)
        finally:
            pkg_resources.cleanup_resources()

    def _rename_placeholder_images(self) -> None:
        """
        Keep the correct placeholder image based on if dark mode is selected, and remove the other one.
        """
        image_dir_path = self.directory_path / "template" / "static" / "images"
        if self.dark:
            os.rename(image_dir_path / "placeholder_white.png", image_dir_path / "placeholder.png")
            os.remove(image_dir_path / "placeholder_black.png")
        else:
            os.rename(image_dir_path / "placeholder_black.png", image_dir_path / "placeholder.png")
            os.remove(image_dir_path / "placeholder_white.png")

    def _change_colors_in_settings_to_dark_mode(self) -> None:

        with open(self.directory_path / "settings.yaml") as f:
            settings = oyaml.safe_load(f)

        settings["colors"]["background"] = "#26263C"
        settings["colors"]["text"] = "#dedeee"
        settings["colors"]["placeholder"] = "#dedeee"
        settings["colors"]["header"] = "white"
        settings["colors"]["accent"] = "orange"

        with open(self.directory_path / "settings.yaml", "w") as f:
            oyaml.dump(settings, f)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['simple_homepage']

package_data = \
{'': ['*'],
 'simple_homepage': ['files/*',
                     'files/template/*',
                     'files/template/static/*',
                     'files/template/static/images/*']}

install_requires = \
['Jinja2>=3.1.2,<4.0.0', 'PyYAML>=6.0,<7.0', 'oyaml>=1.0,<2.0']

entry_points = \
{'console_scripts': ['homepage = simple_homepage.cli:cli']}

setup_kwargs = {
    'name': 'simple-homepage',
    'version': '0.0.3',
    'description': 'Create a simple homepage',
    'long_description': '# simple-homepage\n\n[![Release](https://img.shields.io/github/v/release/fpgmaas/simple-homepage)](https://img.shields.io/github/v/release/fpgmaas/simple-homepage)\n[![Build status](https://img.shields.io/github/workflow/status/fpgmaas/simple-homepage/merge-to-main)](https://img.shields.io/github/workflow/status/fpgmaas/simple-homepage/merge-to-main)\n[![Commit activity](https://img.shields.io/github/commit-activity/m/fpgmaas/simple-homepage)](https://img.shields.io/github/commit-activity/m/fpgmaas/simple-homepage)\n[![Docs](https://img.shields.io/badge/docs-gh--pages-blue)](https://fpgmaas.github.io/simple-homepage/)\n[![Code style with black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Imports with isort](https://img.shields.io/badge/%20imports-isort-%231674b1)](https://pycqa.github.io/isort/)\n[![License](https://img.shields.io/github/license/fpgmaas/simple-homepage)](https://img.shields.io/github/license/fpgmaas/simple-homepage)\n\nThis repository helps you create a simple static homepage for your browser.\n\nThe documentation can be found [here](https://fpgmaas.github.io/simple-homepage/).\n\n### Light ([Link to demo](demo/light/homepage.html))\n\n\n<img src="static/screenshot-light.png" alt="Example light homepage" width="500"/>\n\n### Dark ([Link to demo](demo/dark/homepage.html))\n\n<img src="static/screenshot-dark.png" alt="Example dark homepage" width="500"/>\n\n## Quick start\n\nTo get started, first install the package:\n\n```\npip install simple-homepage\n```\n\nThen, navigate to a directory in which you want to create your homepage, and run\n\n```\nhomepage init\n```\n\nor, for the dark version of the homepage:\n\n```\nhomepage init --dark\n```\n\nThen, modify `settings.yaml` to your liking, and run\n\n```\nhomepage build\n```\n\nYour custom homepage is now available under `public/homepage.html`.\n\n\n---\n\nRepository initiated with [fpgmaas/cookiecutter-poetry](https://github.com/fpgmaas/cookiecutter-poetry).\n',
    'author': 'Florian Maas',
    'author_email': 'ffpgmaas@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/fpgmaas/simple-homepage',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)
